@@ -194,30 +194,6 @@ public abstract class Combiner {
         return COMBINERS.clone();
     }
 
-    /**
-     * Returns a BinList implementation suitable for a given number of
-     * bins and a given combiner.
-     * This may return an implementation based on a hash, or an array,
-     * or some combination.
-     *
-     * @param  combiner  combiner
-     * @param  size    maximum number of bins
-     */
-    public static BinList createDefaultBinList( Combiner combiner, long size ) {
-        if ( size < 1e6 ) {
-            BinList binList = combiner.createArrayBinList( (int) size );
-            if ( binList != null ) {
-                return binList;
-            }
-        }
-        if ( size < Integer.MAX_VALUE ) {
-            return new AdaptiveBinList( (int) size, combiner, 8 );
-        }
-        else {
-            return new HashBinList( size, combiner );
-        }
-    }
-
     /**
      * Utility method to return a string describing the content of a ValueInfo.
      * It will come up with something, even if the description member is empty.
@@ -341,6 +317,17 @@ public abstract class Combiner {
          */
         void submit( double datum );
 
+        /**
+         * Combines the content of a given container with this one.
+         * The effect is the same as if all the data submitted to the
+         * supplied container had in fact been submitted to this one.
+         * The other container is assumed to be of the same type as this one;
+         * if not, a ClassCastException is likely to occur.
+         *
+         * @param   container  other container, of the same type as this one
+         */
+        void add( Container container );
+
         /**
          * Returns the combined result of all the values submitted so far.
          * In general, if no values have been submitted,
@@ -365,24 +352,7 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final int[] counts = new int[ size ];
-            final double[] sums = new double[ size ];
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double value ) {
-                    counts[ index ]++;
-                    sums[ index ] += value;
-                }
-                public double getBinResultInt( int index ) {
-                    int count = counts[ index ];
-                    return count == 0 ? Double.NaN
-                                      : sums[ index ] / (double) count;
-                }
-                public void copyBin( int index, Container bin ) {
-                    MeanContainer container = (MeanContainer) bin;
-                    counts[ index ] = container.count_;
-                    sums[ index ] = container.sum_;
-                }
-            };
+            return new MeanBinList( size, this );
         }
 
         public Container createContainer() {
@@ -399,6 +369,38 @@ public abstract class Combiner {
             return info;
         }
 
+        /**
+         * ArrayBinList subclass for MeanCombiner.
+         */
+        private static class MeanBinList extends ArrayBinList {
+            final int[] counts_;
+            final double[] sums_;
+            MeanBinList( int size, MeanCombiner combiner ) {
+                super( size, combiner );
+                counts_ = new int[ size ];
+                sums_ = new double[ size ];
+            }
+            public void submitToBinInt( int index, double value ) {
+                counts_[ index ]++;
+                sums_[ index ] += value;
+            }
+            public double getBinResultInt( int index ) {
+                int count = counts_[ index ];
+                return count == 0 ? Double.NaN
+                                  : sums_[ index ] / (double) count;
+            }
+            public void copyBin( int index, Container bin ) {
+                MeanContainer container = (MeanContainer) bin;
+                counts_[ index ] = container.count_;
+                sums_[ index ] = container.sum_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                MeanBinList meanOther = (MeanBinList) other;
+                counts_[ index ] += meanOther.counts_[ index ];
+                sums_[ index ] += meanOther.sums_[ index ];
+            }
+        }
+
         /**
          * Container that holds a count and a sum.
          * Note this is a static class to keep memory usage down
@@ -411,6 +413,11 @@ public abstract class Combiner {
                 count_++;
                 sum_ += datum;
             }
+            public void add( Container other ) {
+                MeanContainer meanOther = (MeanContainer) other;
+                count_ += meanOther.count_;
+                sum_ += meanOther.sum_;
+            }
             public double getCombinedValue() {
                 return count_ == 0 ? Double.NaN : sum_ / (double) count_;
             }
@@ -474,26 +481,7 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final int[] counts = new int[ size ];
-            final double[] sum1s = new double[ size ];
-            final double[] sum2s = new double[ size ];
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double value ) {
-                    counts[ index ]++;
-                    sum1s[ index ] += value;
-                    sum2s[ index ] += value * value;
-                }
-                public double getBinResultInt( int index ) {
-                    return getStdev( isSampleStdev_, counts[ index ],
-                                     sum1s[ index ], sum2s[ index ] );
-                }
-                public void copyBin( int index, Container bin ) {
-                    StdevContainer container = (StdevContainer) bin;
-                    counts[ index ] = container.count_;
-                    sum1s[ index ] = container.sum1_;
-                    sum2s[ index ] = container.sum2_;
-                }
-            };
+            return new StdevBinList( size, this );
         }
 
         public Container createContainer() {
@@ -515,6 +503,44 @@ public abstract class Combiner {
             return info;
         }
 
+        /**
+         * ArrayBinList subclass for StdevCombiner.
+         */
+        private static class StdevBinList extends ArrayBinList {
+            final boolean isSampleStdev_;
+            final int[] counts_;
+            final double[] sum1s_;
+            final double[] sum2s_;
+            StdevBinList( int size, StdevCombiner combiner ) {
+                super( size, combiner );
+                isSampleStdev_ = combiner.isSampleStdev_;
+                counts_ = new int[ size ];
+                sum1s_ = new double[ size ];
+                sum2s_ = new double[ size ];
+            }
+            public void submitToBinInt( int index, double value ) {
+                counts_[ index ]++;
+                sum1s_[ index ] += value;
+                sum2s_[ index ] += value * value;
+            }
+            public double getBinResultInt( int index ) {
+                return getStdev( isSampleStdev_, counts_[ index ],
+                                 sum1s_[ index ], sum2s_[ index ] );
+            }
+            public void copyBin( int index, Container bin ) {
+                StdevContainer container = (StdevContainer) bin;
+                counts_[ index ] = container.count_;
+                sum1s_[ index ] = container.sum1_;
+                sum2s_[ index ] = container.sum2_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                StdevBinList stdevOther = (StdevBinList) other;
+                counts_[ index ] += stdevOther.counts_[ index ];
+                sum1s_[ index ] += stdevOther.sum1s_[ index ];
+                sum2s_[ index ] += stdevOther.sum2s_[ index ];
+            }
+        }
+
         /**
          * Partial Container implementation for calculating
          * standard deviation-like quantities.
@@ -528,6 +554,12 @@ public abstract class Combiner {
                 sum1_ += datum;
                 sum2_ += datum * datum;
             }
+            public void add( Container other ) {
+                StdevContainer stdevOther = (StdevContainer) other;
+                count_ += stdevOther.count_;
+                sum1_ += stdevOther.sum1_;
+                sum2_ += stdevOther.sum2_;
+            }
         }
 
         /**
@@ -564,7 +596,7 @@ public abstract class Combiner {
          * @return  standard deviation, or NaN if insufficient data
          */
         private static double getStdev( boolean isSampleStdev, int count,
-                                           double sum1, double sum2 ) {
+                                        double sum1, double sum2 ) {
             if ( count < ( isSampleStdev ? 2 : 1 ) ) {
                 return Double.NaN;
             }
@@ -594,26 +626,39 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final int[] counts = new int[ size ];
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double value ) {
-                    counts[ index ]++;
-                }
-                public double getBinResultInt( int index ) {
-                    int count = counts[ index ];
-                    return count == 0 ? Double.NaN : count;
-                }
-                public void copyBin( int index, Container bin ) {
-                    CountContainer container = (CountContainer) bin;
-                    counts[ index ] = container.count_;
-                }
-            };
+            return new AbstractCountBinList( size, this );
         }
 
         public Container createContainer() {
             return new CountContainer();
         }
 
+        /**
+         * ArrayBinList subclass for AbstractCountCombiner.
+         */
+        static class AbstractCountBinList extends ArrayBinList {
+            final int[] counts_;
+            AbstractCountBinList( int size, AbstractCountCombiner combiner ) {
+                super( size, combiner );
+                counts_ = new int[ size ];
+            }
+            public void submitToBinInt( int index, double value ) {
+                counts_[ index ]++;
+            }
+            public double getBinResultInt( int index ) {
+                int count = counts_[ index ];
+                return count == 0 ? Double.NaN : count;
+            }
+            public void copyBin( int index, Container bin ) {
+                CountContainer container = (CountContainer) bin;
+                counts_[ index ] = container.count_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                AbstractCountBinList countOther = (AbstractCountBinList) other;
+                counts_[ index ] += countOther.counts_[ index ];
+            }
+        }
+
         /**
          * Container that holds a count.
          * Note this is a static class to keep memory usage down
@@ -624,6 +669,10 @@ public abstract class Combiner {
             public void submit( double datum ) {
                 count_++;
             }
+            public void add( Container other ) {
+                CountContainer countOther = (CountContainer) other;
+                count_ += countOther.count_;
+            }
             public double getCombinedValue() {
                 return count_ == 0 ? Double.NaN : count_;
             }
@@ -701,26 +750,41 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final double[] sums = new double[ size ];
-            Arrays.fill( sums, Double.NaN );
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double datum ) {
-                    sums[ index ] = combineSum( sums[ index ], datum );
-                }
-                public double getBinResultInt( int index ) {
-                    return sums[ index ];
-                }
-                public void copyBin( int index, Container bin ) {
-                    SumContainer container = (SumContainer) bin;
-                    sums[ index ] = container.sum_;
-                }
-            };
+            return new AbstractSumBinList( size, this );
         }
 
         public Container createContainer() {
             return new SumContainer();
         }
 
+        /**
+         * ArrayBinList subclass for AbstractSumCombiner.
+         */
+        static class AbstractSumBinList extends ArrayBinList {
+            final double[] sums_;
+            AbstractSumBinList( int size, AbstractSumCombiner combiner ) {
+                super( size, combiner );
+                sums_ = new double[ size ];
+                Arrays.fill( sums_, Double.NaN );
+            }
+            public void submitToBinInt( int index, double datum ) {
+                sums_[ index ] = combineSum( sums_[ index ], datum );
+            }
+            public double getBinResultInt( int index ) {
+                return sums_[ index ];
+            }
+            public void copyBin( int index, Container bin ) {
+                SumContainer container = (SumContainer) bin;
+                sums_[ index ] = container.sum_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                double otherSum = ((AbstractSumBinList) other).sums_[ index ];
+                if ( ! Double.isNaN( otherSum ) ) {
+                    sums_[ index ] = combineSum( sums_[ index ], otherSum );
+                }
+            }
+        }
+
         /**
          * Container that holds a sum.
          * Note this is a static class to keep memory usage down
@@ -731,6 +795,12 @@ public abstract class Combiner {
             public void submit( double datum ) {
                 sum_ = combineSum( sum_, datum );
             }
+            public void add( Container other ) {
+                double otherSum = ((SumContainer) other).sum_;
+                if ( ! Double.isNaN( otherSum ) ) {
+                    sum_ = combineSum( sum_, otherSum );
+                }
+            }
             public double getCombinedValue() {
                 return sum_;
             }
@@ -810,20 +880,7 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final double[] mins = new double[ size ];
-            Arrays.fill( mins, Double.NaN );
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double datum ) {
-                    mins[ index ] = combineMin( mins[ index ], datum );
-                }
-                public double getBinResultInt( int index ) {
-                    return mins[ index ];
-                }
-                public void copyBin( int index, Container bin ) {
-                    MinContainer container = (MinContainer) bin;
-                    mins[ index ] = container.min_;
-                }
-            };
+            return new MinBinList( size, this );
         }
 
         public Container createContainer() {
@@ -842,6 +899,34 @@ public abstract class Combiner {
             return info;
         }
 
+        /**
+         * ArrayBinList subclass for MinCombiner.
+         */
+        private static class MinBinList extends ArrayBinList {
+            final double[] mins_;
+            MinBinList( int size, MinCombiner combiner ) {
+                super( size, combiner );
+                mins_ = new double[ size ];
+                Arrays.fill( mins_, Double.NaN );
+            }
+            public void submitToBinInt( int index, double datum ) {
+                mins_[ index ] = combineMin( mins_[ index ], datum );
+            }
+            public double getBinResultInt( int index ) {
+                return mins_[ index ];
+            }
+            public void copyBin( int index, Container bin ) {
+                MinContainer container = (MinContainer) bin;
+                mins_[ index ] = container.min_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                double otherMin = ((MinBinList) other).mins_[ index ];
+                if ( ! Double.isNaN( otherMin ) ) {
+                    mins_[ index ] = combineMin( mins_[ index ], otherMin );
+                }
+            }
+        }
+
         /**
          * Container that accumulates a minimum.
          * Note this is a static class to keep memory usage down
@@ -852,6 +937,12 @@ public abstract class Combiner {
             public void submit( double datum ) {
                 min_ = combineMin( min_, datum );
             }
+            public void add( Container other ) {
+                double otherMin = ((MinContainer) other).min_;
+                if ( ! Double.isNaN( otherMin ) ) {
+                    min_ = combineMin( min_, otherMin );
+                }
+            }
             public double getCombinedValue() {
                 return min_;
             }
@@ -885,20 +976,7 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final double[] maxs = new double[ size ];
-            Arrays.fill( maxs, Double.NaN );
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double datum ) {
-                    maxs[ index ] = combineMax( maxs[ index ], datum );
-                }
-                public double getBinResultInt( int index ) {
-                    return maxs[ index ];
-                }
-                public void copyBin( int index, Container bin ) {
-                    MaxContainer container = (MaxContainer) bin;
-                    maxs[ index ] = container.max_;
-                }
-            };
+            return new MaxBinList( size, this );
         }
 
         public Container createContainer() {
@@ -917,6 +995,34 @@ public abstract class Combiner {
             return info;
         }
 
+        /**
+         * ArrayBinList subclass for StdevCombiner.
+         */
+        private static class MaxBinList extends ArrayBinList {
+            final double[] maxs_;
+            MaxBinList( int size, MaxCombiner combiner ) {
+                super( size, combiner );
+                maxs_ = new double[ size ];
+                Arrays.fill( maxs_, Double.NaN );
+            }
+            public void submitToBinInt( int index, double datum ) {
+                maxs_[ index ] = combineMax( maxs_[ index ], datum );
+            }
+            public double getBinResultInt( int index ) {
+                return maxs_[ index ];
+            }
+            public void copyBin( int index, Container bin ) {
+                MaxContainer container = (MaxContainer) bin;
+                maxs_[ index ] = container.max_;
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                double otherMax = ((MaxBinList) other).maxs_[ index ];
+                if ( ! Double.isNaN( otherMax ) ) {
+                    maxs_[ index ] = combineMax( maxs_[ index ], otherMax );
+                }
+            }
+        }
+
         /**
          * Container that accumulates a maximum.
          * Note this is a static class to keep memory usage down
@@ -927,6 +1033,12 @@ public abstract class Combiner {
             public void submit( double datum ) {
                 max_ = combineMax( max_, datum );
             }
+            public void add( Container other ) {
+                double otherMax = ((MaxContainer) other).max_;
+                if ( ! Double.isNaN( otherMax ) ) {
+                    max_ = combineMax( max_, otherMax );
+                }
+            }
             public double getCombinedValue() {
                 return max_;
             }
@@ -948,19 +1060,7 @@ public abstract class Combiner {
         }
 
         public ArrayBinList createArrayBinList( int size ) {
-            final BitSet mask = new BitSet();
-            return new ArrayBinList( size, this ) {
-                public void submitToBinInt( int index, double datum ) {
-                    mask.set( index );
-                }
-                public double getBinResultInt( int index ) {
-                    return mask.get( index ) ? 1 : Double.NaN;
-                }
-                public void copyBin( int index, Container bin ) {
-                    HitContainer container = (HitContainer) bin;
-                    mask.set( index, container.hit_ );
-                }
-            };
+            return new HitBinList( size, this );
         }
 
         public Container createContainer() {
@@ -973,11 +1073,43 @@ public abstract class Combiner {
                                          "1 if bin contains data, 0 if not" );
         }
 
+        /**
+         * ArrayBinList subclass for HitCombiner.
+         */
+        private static class HitBinList extends ArrayBinList {
+            final BitSet mask_;
+            HitBinList( int size, HitCombiner combiner ) {
+                super( size, combiner );
+                mask_ = new BitSet();
+            }
+            public void submitToBinInt( int index, double datum ) {
+                mask_.set( index );
+            }
+            public double getBinResultInt( int index ) {
+                return mask_.get( index ) ? 1 : Double.NaN;
+            }
+            public void copyBin( int index, Container bin ) {
+                HitContainer container = (HitContainer) bin;
+                mask_.set( index, container.hit_ );
+            }
+            public void addBin( int index, ArrayBinList other ) {
+                if ( ((HitBinList) other).mask_.get( index ) ) {
+                    mask_.set( index );
+                }
+            }
+        }
+
         private static class HitContainer implements Container {
             boolean hit_;
             public void submit( double datum ) {
                 hit_ = true;
             }
+            public void add( Container other ) {
+                boolean otherHit = ((HitContainer) other).hit_;
+                if ( otherHit ) {
+                    hit_ = true;
+                }
+            }
             public double getCombinedValue() {
                 return hit_ ? 1 : Double.NaN;
             }
