@@ -59,6 +59,7 @@ public enum FixedCode implements ReportCode {
     E_DFIO( "Duff query failed" ),
     E_DFSF( "Duff query result parse failure" ),
     E_DNST( "Missing QUERY_STATUS from duff query" ),
+    E_DQUM( "QUERY_STATUS inconsistency" ),
     E_DQUS( "Unknown value for QUERY_STATUS" ),
     E_DSUC( "Non-error return from duff query" ),
     E_ELDF( "Wrong top-level element" ),
@@ -250,6 +251,7 @@ public enum FixedCode implements ReportCode {
     I_CJOB( "Job created" ),
     I_CURL( "Reading capability metadata" ),
     I_DMID( "Data model declared" ),
+    I_DQUR( "QUERY_STATUS redundancy" ),
     I_DUFF( "Duff query" ),
     I_EURL( "Reading examples document" ),
     I_EXA2( "Assume default query language" ),
